@@ -10,13 +10,13 @@ Usage:
   list
   delete n
 """
-__version__ = "2.62"
-__author__ = "Aaron Swartz (me@aaronsw.com)"
-__copyright__ = "(C) 2004 Aaron Swartz. GNU GPL 2."
+__version__ = "2.65"
+__author__ = "Lindsey Smith (rss2email@aaronsw.com)"
+__copyright__ = "(C) 2004 Aaron Swartz. GNU GPL 2 or 3."
 ___contributors__ = ["Dean Jackson", "Brian Lalor", "Joey Hess", 
                      "Matej Cepl", "Martin 'Joey' Schulze", 
                      "Marcel Ackermann (http://www.DreamFlasher.de)", 
-                     "Lindsey Smith (lindsey.smith@gmail.com)" ]
+                     "Lindsey Smith", "Aaron Swartz (original author)" ]
 
 ### Vaguely Customizable Options ###
 
@@ -48,8 +48,10 @@ DATE_HEADER_ORDER = ('modified', 'issued', 'created')
 # 1: Apply Q-P conversion (required for some MUAs).
 # 0: Send message in 8-bits.
 # http://cr.yp.to/smtp/8bitmime.html
+#DEPRECATED 
 QP_REQUIRED = 0
-
+#DEPRECATED 
+	
 # 1: Name feeds as they're being processed.
 # 0: Keep quiet.
 VERBOSE = 0
@@ -117,7 +119,7 @@ def send(sender, recipient, subject, body, contenttype, extraheaders=None, smtps
 	for body_charset in CHARSET_LIST:
 	    try:
 	        body.encode(body_charset)
-	    except UnicodeError:
+	    except (UnicodeError, LookupError):
 	        pass
 	    else:
 	        break
@@ -140,16 +142,19 @@ def send(sender, recipient, subject, body, contenttype, extraheaders=None, smtps
 	msg['To'] = formataddr((recipient_name, recipient_addr))
 	msg['Subject'] = Header(unicode(subject), header_charset)
 	for hdr in extraheaders.keys():
-		msg[hdr] = Header(unicode(extraheaders[hdr], header_charset))
+		try:
+			msg[hdr] = Header(unicode(extraheaders[hdr], header_charset))
+		except:
+			msg[hdr] = Header(extraheaders[hdr])
 		
 	fromhdr = formataddr((sender_name, sender_addr))
 	msg['From'] = fromhdr
 		
 	msg_as_string = msg.as_string()
-	if QP_REQUIRED:
-		ins, outs = SIO(msg_as_string), SIO()
-		mimify.mimify(ins, outs)
-		msg_as_string = outs.getvalue()
+#DEPRECATED 	if QP_REQUIRED:
+#DEPRECATED 		ins, outs = SIO(msg_as_string), SIO()
+#DEPRECATED 		mimify.mimify(ins, outs)
+#DEPRECATED 		msg_as_string = outs.getvalue()
     		
 	if SMTP_SEND:
 		if not smtpserver: 
@@ -185,10 +190,27 @@ def send(sender, recipient, subject, body, contenttype, extraheaders=None, smtps
 		return smtpserver
 
 	else:
-		i, o = os.popen2(["/usr/sbin/sendmail", recipient])
-		i.write(msg_as_string)
-		i.close(); o.close()
-		del i, o
+		try:
+			p = subprocess.Popen(["/usr/sbin/sendmail", recipient], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+			p.communicate(msg_as_string)
+			status = p.returncode
+			assert status != None, "just a sanity check"
+			if status != 0:
+				print >>warn, ""
+				print >>warn, ('Fatal error: sendmail exited with code %s' % status)
+				sys.exit(1)
+		except:
+			print '''Error attempting to send email via sendmail. Possibly you need to configure your config.py to use a SMTP server? Please refer to the rss2email documentation or website (http://rss2email.infogami.com) for complete documentation of config.py. The options below may suffice for configuring email:
+# 1: Use SMTP_SERVER to send mail.
+# 0: Call /usr/sbin/sendmail to send mail.
+SMTP_SEND = 0
+
+SMTP_SERVER = "smtp.yourisp.net:25"
+AUTHREQUIRED = 0 # if you need to use SMTP AUTH set to 1
+SMTP_USER = 'username'  # for SMTP AUTH, set SMTP username here
+SMTP_PASS = 'password'  # for SMTP AUTH, set SMTP password here
+'''
+			sys.exit(1)
 		return None
 
 ## html2text options ##
@@ -206,26 +228,44 @@ BODY_WIDTH = 0
 
 # Read options from config file if present.
 import sys
-sys.path.append(".")
+sys.path.insert(0,".")
 try:
 	from config import *
 except:
 	pass
+
+warn = sys.stderr
 	
+if QP_REQUIRED:
+	print >>warn, "QP_REQUIRED has been deprecated in rss2email."
+
 ### Import Modules ###
 
-import cPickle as pickle, md5, time, os, traceback, urllib2, sys, types
+import cPickle as pickle, time, os, traceback, urllib2, sys, types, subprocess
+hash = ()
+try:
+	import hashlib
+	hash = hashlib.md5
+except ImportError:
+	import md5
+	hash = md5.new
+
 unix = 0
 try:
 	import fcntl
-	unix = 1
+# A pox on SunOS file locking methods	
+	if (sys.platform.find('sunos') == -1): 
+		unix = 1
 except:
 	pass
 		
 import socket; socket_errors = []
 for e in ['error', 'gaierror']:
 	if hasattr(socket, e): socket_errors.append(getattr(socket, e))
-import mimify; from StringIO import StringIO as SIO; mimify.CHARSET = 'utf-8'
+
+#DEPRECATED import mimify 
+#DEPRECATED from StringIO import StringIO as SIO 
+#DEPRECATED mimify.CHARSET = 'utf-8'
 
 import feedparser
 feedparser.USER_AGENT = "rss2email/"+__version__+ " +http://www.aaronsw.com/2002/rss2email/"
@@ -275,8 +315,6 @@ def timelimit(timeout, function):
 #    return internal
     
 
-warn = sys.stderr
-
 def isstr(f): return isinstance(f, type('')) or isinstance(f, type(u''))
 def ishtml(t): return type(t) is type(())
 def contains(a,b): return a.find(b) != -1
@@ -284,18 +322,6 @@ def unu(s): # I / freakin' hate / that unicode
 	if type(s) is types.UnicodeType: return s.encode('utf-8')
 	else: return s
 
-def quote822(s):
-	"""Quote names in email according to RFC822."""
-	return '"' + unu(s).replace("\\", "\\\\").replace('"', '\\"') + '"'
-
-def header7bit(s):
-	"""QP_CORRUPT headers."""
-	#return mimify.mime_encode_header(s + ' ')[:-1]
-	# XXX due to mime_encode_header bug
-	import re
-	p = re.compile('=\n([^ \t])');
-	return p.sub(r'\1', mimify.mime_encode_header(s + ' ')[:-1])
-
 ### Parsing Utilities ###
 
 def getContent(entry, HTMLOK=0):
@@ -342,9 +368,9 @@ def getID(entry):
 		if 'id' in entry and entry.id: return entry.id
 
 	content = getContent(entry)
-	if content and content != "\n": return md5.new(unu(content)).hexdigest()
+	if content and content != "\n": return hash(unu(content)).hexdigest()
 	if 'link' in entry: return entry.link
-	if 'title' in entry: return md5.new(unu(entry.title)).hexdigest()
+	if 'title' in entry: return hash(unu(entry.title)).hexdigest()
 
 def getName(r, entry):
 	"""Get the best name."""
@@ -411,12 +437,17 @@ def load(lock=1):
 		print "Feedfile could not be opened: %s" % e
 		sys.exit(1)
 	feeds = pickle.load(feedfileObject)
+	
 	if lock:
-		if unix: fcntl.flock(feedfileObject.fileno(), fcntl.LOCK_EX)
+		locktype = 0
+		if unix:
+			locktype = fcntl.LOCK_EX
+			fcntl.flock(feedfileObject.fileno(), locktype)
 		#HACK: to deal with lock caching
 		feedfileObject = open(feedfile, 'r')
 		feeds = pickle.load(feedfileObject)
-		if unix: fcntl.flock(feedfileObject.fileno(), fcntl.LOCK_EX)
+		if unix: 
+			fcntl.flock(feedfileObject.fileno(), locktype)
 
 	return feeds, feedfileObject
 
@@ -455,6 +486,7 @@ def add(*args):
 
 def run(num=None):
 	feeds, feedfileObject = load()
+	smtpserver = None
 	try:
 		# We store the default to address as the first item in the feeds list.
 		# Here we take it out and save it for later.
@@ -463,17 +495,17 @@ def run(num=None):
 		else: ifeeds = feeds
 		
 		if num: ifeeds = [feeds[num]]
-		
-		smtpserver = None
+		feednum = 0
 		
 		for f in ifeeds:
 			try: 
-				if VERBOSE: print >>warn, "I: Processing", f.url
+				feednum += 1
+				if VERBOSE: print >>warn, 'I: Processing [%d] "%s"' % (feednum, f.url)
 				r = {}
 				try:
 					r = timelimit(FEED_TIMEOUT, parse)(f.url, f.etag, f.modified)
 				except TimeoutError:
-					print >>warn, 'W: feed "%s" timed out' % f.url
+					print >>warn, 'W: feed [%d] "%s" timed out' % (feednum, f.url)
 					continue
 				
 				# Handle various status conditions, as required
@@ -491,42 +523,42 @@ def run(num=None):
 				exc_type = r.get("bozo_exception", Exception()).__class__
 				if http_status != 304 and not r.get('version', ''):
 					if http_status not in [200, 302]: 
-						print >>warn, "W: error", http_status, f.url
+						print >>warn, "W: error %d [%d] %s" % (http_status, feednum, f.url)
 
 					elif contains(http_headers.get('content-type', 'rss'), 'html'):
-						print >>warn, "W: looks like HTML", f.url
+						print >>warn, "W: looks like HTML [%d] %s"  % (feednum, f.url)
 
 					elif http_headers.get('content-length', '1') == '0':
-						print >>warn, "W: empty page", f.url
+						print >>warn, "W: empty page [%d] %s" % (feednum, f.url)
 
 					elif hasattr(socket, 'timeout') and exc_type == socket.timeout:
-						print >>warn, "W: timed out on", f.url
+						print >>warn, "W: timed out on [%d] %s" % (feednum, f.url)
 					
 					elif exc_type == IOError:
-						print >>warn, "W:", r.bozo_exception, f.url
+						print >>warn, 'W: "%s" [%d] %s' % (r.bozo_exception, feednum, f.url)
 					
 					elif hasattr(feedparser, 'zlib') and exc_type == feedparser.zlib.error:
-						print >>warn, "W: broken compression", f.url
+						print >>warn, "W: broken compression [%d] %s" % (feednum, f.url)
 					
 					elif exc_type in socket_errors:
 						exc_reason = r.bozo_exception.args[1]
-						print >>warn, "W:", exc_reason, f.url
+						print >>warn, "W: %s [%d] %s" % (exc_reason, feednum, f.url)
 
 					elif exc_type == urllib2.URLError:
 						if r.bozo_exception.reason.__class__ in socket_errors:
 							exc_reason = r.bozo_exception.reason.args[1]
 						else:
 							exc_reason = r.bozo_exception.reason
-						print >>warn, "W:", exc_reason, f.url
+						print >>warn, "W: %s [%d] %s" % (exc_reason, feednum, f.url)
 					
 					elif exc_type == AttributeError:
-						print >>warn, "W:", r.bozo_exception, f.url
+						print >>warn, "W: %s [%d] %s" % (r.bozo_exception, feednum, f.url)
 					
 					elif exc_type == KeyboardInterrupt:
 						raise r.bozo_exception
 						
 					elif r.bozo:
-						print >>warn, 'E: error in "%s" feed (%s)' % (f.url, r.get("bozo_exception", "can't process"))
+						print >>warn, 'E: error in [%d] "%s" feed (%s)' % (feednum, f.url, r.get("bozo_exception", "can't process"))
 
 					else:
 						print >>warn, "=== SEND THE FOLLOWING TO rss2email@aaronsw.com ==="
@@ -617,8 +649,10 @@ def run(num=None):
 						content += '\n<p class="footer">URL: <a href="'+link+'">'+link+'</a>'
 						if hasattr(entry,'enclosures'):
 							for enclosure in entry.enclosures:
-								if enclosure.url != "":
+								if (hasattr(enclosure, 'url') and enclosure.url != ""):
 									content += ('<br/>Enclosure: <a href="'+unu(enclosure.url)+'">'+unu(enclosure.url)+"</a>\n")
+								if (hasattr(enclosure, 'src') and enclosure.src != ""):
+									content += ('<br/>Enclosure: <a href="'+unu(enclosure.src)+'">'+unu(enclosure.src)+'</a><br/><img src="'+unu(enclosure.src)+'"\n')
 						content += '</p></div>\n'
 						content += "\n\n</body></html>"
 					else:	
@@ -746,5 +780,3 @@ if __name__ == '__main__':
 		print
 		print __doc__
 
-
-
